@@ -1,6 +1,5 @@
 use ang::Angle;
 use pyo3::prelude::*;
-use pyo3::PyNumberProtocol;
 
 #[pyclass(name = "Angle", module = "si_units")]
 #[derive(Clone, Copy)]
@@ -18,33 +17,30 @@ impl From<PyAngle> for Angle {
     }
 }
 
-#[pyproto]
-impl pyo3::class::basic::PyObjectProtocol for PyAngle {
+#[pymethods]
+impl PyAngle {
     fn __repr__(&self) -> PyResult<String> {
         Ok(self.0.to_string())
     }
-}
 
-#[pyproto]
-impl PyNumberProtocol for PyAngle {
-    fn __add__(lhs: PyRef<'p, Self>, rhs: &PyAny) -> PyResult<PyObject> {
+    fn __add__(&self, rhs: &PyAny) -> PyResult<PyObject> {
         Python::with_gil(|py| {
             rhs.extract::<PyAngle>()
-                .map(|r| Ok(PyCell::new(py, Self(lhs.0 + r.0))?.to_object(py)))?
+                .map(|r| Ok(PyCell::new(py, Self(self.0 + r.0))?.to_object(py)))?
         })
     }
 
-    fn __sub__(lhs: PyRef<'p, Self>, rhs: &PyAny) -> PyResult<PyObject> {
+    fn __sub__(&self, rhs: &PyAny) -> PyResult<PyObject> {
         Python::with_gil(|py| {
             rhs.extract::<PyAngle>()
-                .map(|r| Ok(PyCell::new(py, Self(lhs.0 - r.0))?.to_object(py)))?
+                .map(|r| Ok(PyCell::new(py, Self(self.0 - r.0))?.to_object(py)))?
         })
     }
 
-    fn __mul__(lhs: PyRef<'p, Self>, rhs: &PyAny) -> PyResult<PyObject> {
+    fn __mul__(&self, rhs: &PyAny) -> PyResult<PyObject> {
         Python::with_gil(|py| {
             rhs.extract::<f64>()
-                .map(|r| Ok(PyCell::new(py, Self(lhs.0 * r))?.to_object(py)))?
+                .map(|r| Ok(PyCell::new(py, Self(self.0 * r))?.to_object(py)))?
         })
     }
 
@@ -55,10 +51,10 @@ impl PyNumberProtocol for PyAngle {
         })
     }
 
-    fn __truediv__(lhs: PyRef<'p, Self>, rhs: &PyAny) -> PyResult<PyObject> {
+    fn __truediv__(&self, rhs: &PyAny) -> PyResult<PyObject> {
         Python::with_gil(|py| {
             rhs.extract::<f64>()
-                .map(|r| Ok(PyCell::new(py, Self(lhs.0 / r))?.to_object(py)))?
+                .map(|r| Ok(PyCell::new(py, Self(self.0 / r))?.to_object(py)))?
         })
     }
 
